@@ -428,9 +428,9 @@ universal_argument (int keytype, int xarg)
 
   for (;;)
     {
-      astr_cat_char (as, '-');	/* Add the '-' character. */
+      astr_cat_char (as, '-'); /* Add the '-' character. */
       key = do_binding_completion (as);
-      astr_truncate (as, -1);	/* Remove the '-' character. */
+      astr_truncate (as, astr_len (as) - 1); /* Remove the '-' character. */
 
       /* Cancelled. */
       if (key == KBD_CANCEL)
@@ -586,7 +586,7 @@ edit_tab_line (Line ** lp, size_t lineno, size_t offset, size_t size,
   col = 0;
   for (i = 0; i < offset; i++)
     {
-      if (*astr_char ((*lp)->text, (ptrdiff_t) i) == '\t')
+      if (*astr_char ((*lp)->text, i) == '\t')
         col |= t - 1;
       ++col;
     }
@@ -771,12 +771,12 @@ END_DEFUN
                                ISCLOSEBRACKETCHAR (c))
 #define PRECEDINGQUOTEDQUOTE(c) (c == '\\' \
     && cur_bp->pt.o + 1 < astr_len (cur_bp->pt.p->text) \
-    && ((*astr_char (cur_bp->pt.p->text, (ptrdiff_t) (cur_bp->pt.o + 1)) == '\"') || \
-        (*astr_char (cur_bp->pt.p->text, (ptrdiff_t) (cur_bp->pt.o + 1)) == '\'')))
+    && ((*astr_char (cur_bp->pt.p->text, cur_bp->pt.o + 1) == '\"') || \
+        (*astr_char (cur_bp->pt.p->text, cur_bp->pt.o + 1) == '\'')))
 #define FOLLOWINGQUOTEDQUOTE(c) (c == '\\' \
     && cur_bp->pt.o + 1 < astr_len (cur_bp->pt.p->text) \
-    && ((*astr_char (cur_bp->pt.p->text, (ptrdiff_t) (cur_bp->pt.o + 1)) == '\"') || \
-        (*astr_char (cur_bp->pt.p->text, (ptrdiff_t) (cur_bp->pt.o + 1)) == '\'')))
+    && ((*astr_char (cur_bp->pt.p->text, cur_bp->pt.o + 1) == '\"') || \
+        (*astr_char (cur_bp->pt.p->text, cur_bp->pt.o + 1) == '\'')))
 
 static int
 move_sexp (int dir)
@@ -1253,7 +1253,7 @@ setcase_word (int rcase)
 {
   int gotword;
   size_t i, size;
-  char *p;
+  char c;
 
   if (!ISWORDCHAR (following_char ()))
     if (!forward_word () || !backward_word ())
@@ -1261,7 +1261,7 @@ setcase_word (int rcase)
 
   for (i = cur_bp->pt.o;
        i < astr_len (cur_bp->pt.p->text) &&
-         ISWORDCHAR ((int) *astr_char (cur_bp->pt.p->text, (ptrdiff_t) i));
+         ISWORDCHAR ((int) *astr_char (cur_bp->pt.p->text, i));
        i++)
     ;
   size = i - cur_bp->pt.o;
@@ -1271,25 +1271,26 @@ setcase_word (int rcase)
   gotword = false;
   for (gotword = false;
        cur_bp->pt.o < astr_len (cur_bp->pt.p->text) &&
-         ISWORDCHAR ((int) *(p = astr_char (cur_bp->pt.p->text, (ptrdiff_t) cur_bp->pt.o)));
+         ISWORDCHAR ((c = *astr_char (cur_bp->pt.p->text, cur_bp->pt.o)));
        cur_bp->pt.o++, gotword = true)
     {
-      if (isalpha ((int) *p))
+      if (isalpha ((int) c))
         {
           switch (rcase)
             {
             case UPPERCASE:
-              *p = toupper (*p);
+              c = toupper (c);
               break;
             case LOWERCASE:
-              *p = tolower (*p);
+              c = tolower (c);
               break;
             case CAPITALIZE:
-              *p = (gotword ? tolower : toupper) (*p);
+              c = (gotword ? tolower : toupper) (c);
               break;
             default:
               break;
             }
+          astr_nreplace_cstr (cur_bp->pt.p->text, cur_bp->pt.o, 1, &c, 1);
         }
     }
 
@@ -1368,12 +1369,8 @@ setcase_region (int rcase)
     {
       if (i < astr_len (lp->text))
         {
-          if (rcase == UPPERCASE)
-            *astr_char (lp->text, (ptrdiff_t) i) =
-              toupper (*astr_char (lp->text, (ptrdiff_t) i));
-          else
-            *astr_char (lp->text, (ptrdiff_t) i) =
-              tolower (*astr_char (lp->text, (ptrdiff_t) i));
+          char c = (rcase == UPPERCASE ? toupper : tolower) (*astr_char (lp->text, i));
+          astr_nreplace_cstr (lp->text, i, 1, &c, 1);
           ++i;
         }
       else
@@ -1417,9 +1414,9 @@ write_shell_output (va_list ap)
 static bool
 pipe_command (const char *cmd, const char *tempfile, bool insert, bool replace)
 {
-  astr out = astr_new (), s;
-  size_t lines = 0;
-  char *cmdline;
+  astr out = astr_new ();
+  bool more_than_one_line = false;
+  char *cmdline, *eol;
   FILE * pipe;
 
   xasprintf (&cmdline, "%s 2>&1 <%s", cmd, tempfile);
@@ -1431,17 +1428,11 @@ pipe_command (const char *cmd, const char *tempfile, bool insert, bool replace)
     }
   free (cmdline);
 
-  for (;;)
-    {
-      s = astr_fgets (pipe);
-      astr_cat (out, s);
-      astr_delete (s);
-      if (feof (pipe))
-        break;
-      ++lines;
-      astr_cat_char (out, '\n');
-    }
+  out = astr_fread (pipe);
   pclose (pipe);
+  eol = strchr (astr_cstr (out), '\n');
+  if (eol && eol != astr_cstr (out) + astr_len (out) - 1)
+    more_than_one_line = true;
 
   if (astr_len (out) == 0)
     minibuf_write ("(Shell command succeeded with no output)");
@@ -1460,9 +1451,9 @@ pipe_command (const char *cmd, const char *tempfile, bool insert, bool replace)
         }
       else
         {
-          write_temp_buffer ("*Shell Command Output*", lines > 1,
+          write_temp_buffer ("*Shell Command Output*", more_than_one_line,
                              write_shell_output, out);
-          if (lines <= 1)
+          if (!more_than_one_line)
             minibuf_write ("%s", astr_cstr (out));
         }
     }
