@@ -1,8 +1,6 @@
 /* Lisp lists
 
    Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 2001 Scott "Jerry" Lawrence.
-   Copyright (c) 2005 Reuben Thomas.
 
    This file is part of GNU Zile.
 
@@ -24,7 +22,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 le *
