@@ -1,7 +1,6 @@
 /* Undo facility functions
 
-   Copyright (c) 2008 Free Software Foundation, Inc.
-   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004 Sandro Sigala.
+   Copyright (c) 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GNU Zile.
 
@@ -27,7 +26,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "zile.h"
+#include "main.h"
 #include "extern.h"
 
 /*
@@ -164,7 +163,6 @@ Repeat this command to undo more changes.
 
   cur_bp->next_undop = revert_action (cur_bp->next_undop);
   minibuf_write ("Undo!");
-  return leT;
 }
 END_DEFUN
 
